# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-01 16:00
from __future__ import unicode_literals

from collections import defaultdict
from django.db import migrations, models, router


def de_dupe_version_table(apps, schema_editor):
    """
    Removes some duplicate Version models that may have crept into the database and will prevent the
    unique index being added by migration 0004.
    """
    Version = apps.get_model("reversion", "Version")
    keep_version_ids = Version.objects.order_by().values_list(
        # Group by the unique constraint we intend to enforce.
        "revision_id",
        "content_type_id",
        "object_id",
    ).annotate(
        # Add in the most recent id for each duplicate row.
        max_pk=models.Max("pk"),
    ).values_list("max_pk", flat=True)
    # Delete all duplicate versions. Can't do this as a delete with subquery because MySQL doesn't like running a
    # subquery on the table being updated/deleted.
    delete_version_ids = list(Version.objects.exclude(
        pk__in=keep_version_ids,
    ).values_list("pk", flat=True))
    Version.objects.filter(
        pk__in=delete_version_ids,
    ).delete()


def set_version_db(apps, schema_editor):
    """
    Updates the db field in all Version models to point to the correct write
    db for the model.
    """
    Version = apps.get_model("reversion", "Version")
    content_types = Version.objects.order_by().values_list("content_type__app_label", "content_type__model").distinct()
    model_dbs = defaultdict(list)
    for app_label, model_name in content_types:
        model = apps.get_model(app_label, model_name)
        db = router.db_for_write(model)
        model_dbs[db].append((app_label, model_name))
    for db, model_keys in model_dbs.items():
        db_query = models.Q()
        for app_label, model_name in model_keys:
            db_query |= models.Q(
                content_type__app_label=app_label, content_type__model=model_name
            )
        Version.objects.filter(db_query).update(db=db)


class Migration(migrations.Migration):

    dependencies = [
        ('reversion', '0002_auto_20141216_1509'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='revision',
            name='manager_slug',
        ),
        migrations.RemoveField(
            model_name='version',
            name='object_id_int',
        ),
        migrations.AlterField(
            model_name='version',
            name='object_id',
            field=models.CharField(help_text='Primary key of the model under version control.', max_length=191),
        ),
        migrations.AlterField(
            model_name='revision',
            name='date_created',
            field=models.DateTimeField(db_index=True, help_text='The date and time this revision was created.', verbose_name='date created'),
        ),
        migrations.AddField(
            model_name='version',
            name='db',
            field=models.CharField(null=True, help_text='The database the model under version control is stored in.', max_length=191),
        ),
        migrations.RunPython(de_dupe_version_table),
        migrations.RunPython(set_version_db),
    ]
